(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[405],{34376:function(e,t,n){e.exports=n(84142)},75557:function(e,t,n){(window.__NEXT_P=window.__NEXT_P||[]).push(["/",function(){return n(48591)}])},31143:function(e,t,n){"use strict";var i=n(28598),a=n(1887),s=n.n(a);t.Z=()=>(0,i.jsxs)(s(),{children:[(0,i.jsx)("title",{children:"Defog.ai - AI Assistant for Data Analysis"}),(0,i.jsx)("meta",{name:"description",content:"Train your AI data assistant on your own device"}),(0,i.jsx)("meta",{name:"viewport",content:"width=device-width, initial-scale=1"}),(0,i.jsx)("link",{rel:"icon",href:"/favicon.ico"})]})},62429:function(e,t,n){"use strict";var i=n(28598);n(82684);var a=n(79869),s=n(12691),r=n.n(s);t.Z=e=>{let{id:t,children:n}=e,{Content:s,Sider:o}=a.Layout,d=[{key:"manage-database",title:"Manage Database",icon:(0,i.jsx)(r(),{href:"/extract-metadata",children:"\uD83D\uDCBE Manage DB"})},{key:"instruct-model",title:"Instruct Model",icon:(0,i.jsx)(r(),{href:"/instruct-model",children:"\uD83D\uDC68‍\uD83C\uDFEB Instruct Model"})},{key:"query-data",title:"Query Data",icon:(0,i.jsx)(r(),{href:"/query-data",children:"\uD83D\uDD0D Query Data"})}];return(0,i.jsx)(a.Layout,{style:{height:"100vh"},children:(0,i.jsxs)(s,{children:[(0,i.jsx)(o,{style:{height:"100vh",position:"fixed"},children:(0,i.jsx)(a.v2,{style:{width:200,paddingTop:"2em",paddingBottom:"2em"},mode:"inline",selectedKeys:[t],items:d})}),(0,i.jsx)("div",{style:{paddingLeft:240,paddingTop:30,backgroundColor:"#f5f5f5"},children:n})]})})}},48591:function(e,t,n){"use strict";n.r(t);var i=n(28598),a=n(82684),s=n(34376),r=n(31143),o=n(62429),d=n(79869);t.default=()=>{let e=(0,s.useRouter)();return(0,a.useEffect)(()=>{fetch("http://localhost:1235/integration/get_tables_db_creds",{method:"POST",headers:{"Content-Type":"application/json"}}).then(e=>e.json()).then(t=>{t.tables&&t.db_creds?e.push("/query-data"):e.push("/extract-metadata")})},[]),(0,i.jsxs)(i.Fragment,{children:[(0,i.jsx)(r.Z,{}),(0,i.jsxs)(o.Z,{children:[(0,i.jsx)("h1",{style:{paddingBottom:"1em"},children:"Welcome to Defog!"}),(0,i.jsxs)("h3",{children:["Please wait while we log you in and redirect you to the right page... ",(0,i.jsx)(d.yC,{})]})]})]})}}},function(e){e.O(0,[238,774,888,179],function(){return e(e.s=75557)}),_N_E=e.O()}]);