(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[888],{91118:function(n,e,_){(window.__NEXT_P=window.__NEXT_P||[]).push(["/_app",function(){return _(36290)}])},36290:function(n,e,_){"use strict";_.r(e),_.d(e,{default:function(){return App}});var t=_(28598);_(50558);var u=_(82684);let c=u.createContext();function App(n){let{Component:e,pageProps:_}=n,[r,i]=(0,u.useState)({});return(0,t.jsx)(c.Provider,{value:[r,i],children:(0,t.jsx)(e,{..._})})}_(46731)},50558:function(){},46731:function(){}},function(n){var __webpack_exec__=function(e){return n(n.s=e)};n.O(0,[774,179],function(){return __webpack_exec__(91118),__webpack_exec__(84142)}),_N_E=n.O()}]);