self.__BUILD_MANIFEST=function(e){return{__rewrites:{afterFiles:[],beforeFiles:[],fallback:[]},"/":[e,"static/chunks/pages/index-b60f249c1d54d3bf.js"],"/_error":["static/chunks/pages/_error-ee42a9921d95ff81.js"],"/extract-metadata":[e,"static/chunks/pages/extract-metadata-2adf74ad3bcc8699.js"],"/instruct-model":[e,"static/chunks/pages/instruct-model-d040be04cf7f21f2.js"],"/query-data":[e,"static/chunks/pages/query-data-b8197e7950b177eb.js"],sortedPages:["/","/_app","/_error","/extract-metadata","/instruct-model","/query-data"]}}("static/chunks/238-21e16f207d48d221.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();